@@ -2,11 +2,11 @@
 **  Copyright (c) 2004-2007 Sendmail, Inc. and its suppliers.
 **	All rights reserved.
 **
-**  $Id: dk-filter.c,v 1.169 2007/04/08 00:22:44 msk Exp $
+**  $Id: dk-filter.c,v 1.172 2007/05/31 18:59:00 msk Exp $
 */
 
 #ifndef lint
-static char dk_filter_c_id[] = "@(#)$Id: dk-filter.c,v 1.169 2007/04/08 00:22:44 msk Exp $";
+static char dk_filter_c_id[] = "@(#)$Id: dk-filter.c,v 1.172 2007/05/31 18:59:00 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -41,11 +41,6 @@ static char dk_filter_c_id[] = "@(#)$Id: dk-filter.c,v 1.169 2007/04/08 00:22:44
 #include <netdb.h>
 #include <signal.h>
 #include <regex.h>
-#include <openssl/err.h>
-#if _FFR_REPORTINFO
-# include <openssl/bio.h>
-# include <openssl/evp.h>
-#endif /* _FFR_REPORTINFO */
 #if SOLARIS
 # define _PATH_DEVNULL		"/dev/null"
 # ifndef _PATH_SENDMAIL
@@ -55,6 +50,11 @@ static char dk_filter_c_id[] = "@(#)$Id: dk-filter.c,v 1.169 2007/04/08 00:22:44
 # include <paths.h>
 #endif /* SOLARIS */
 
+/* openssl includes */
+#include <openssl/err.h>
+#include <openssl/bio.h>
+#include <openssl/evp.h>
+
 /* sendmail includes */
 #include <sm/cdefs.h>
 #include <sm/string.h>
@@ -290,13 +290,9 @@ static void dkf_cleanup __P((SMFICTX *));
 bool addxhdr;					/* add identifying header? */
 bool hdrlist;					/* signature includes hdrlist */
 bool dolog;					/* syslog */
-#if _FFR_QUARANTINE
 bool quarantine;				/* quarantine failures? */
-#endif /* _FFR_QUARANTINE */
 bool no_i_whine;				/* noted ${i} is undefined */
-#if _FFR_REPORTINFO
 bool send_reports;				/* send failure reports */
-#endif /* _FFR_REPORTINFO */
 #if _FFR_FLUSH_HEADERS
 bool flushheaders;				/* flush old DK headers */
 #endif /* _FFR_FLUSH_HEADERS */
@@ -333,9 +329,7 @@ DB *popdb;					/* POP auth DB */
 #endif /* POPAUTH */
 pthread_mutex_t count_lock;			/* counter lock */
 int thread_count;				/* thread count */
-#if _FFR_REPORTINFO
 pthread_mutex_t popen_lock;			/* popen() lock */
-#endif /* _FFR_REPORTINFO */
 
 #ifdef DEBUG
 static void *fakepriv;				/* fake private space */
@@ -869,7 +863,6 @@ dkf_libstatus(SMFICTX *ctx, char *where, int status)
 		break;
 	}
 
-#ifdef _FFR_LOG_SSL_ERRORS
 	if (ERR_peek_error() != 0 && dolog)
 	{
 		int n;
@@ -893,7 +886,6 @@ dkf_libstatus(SMFICTX *ctx, char *where, int status)
 
 		syslog(LOG_INFO, "%s SSL %s", JOBID(dfc->mctx_jobid), errbuf);
 	}
-#endif /* _FFR_LOG_SSL_ERRORS */
 
 	if (status != DK_STAT_OK)
 		dkf_cleanup(ctx);
@@ -1184,7 +1176,6 @@ dkf_loadkeys(char *file)
 }
 #endif /* _FFR_MULTIPLE_KEYS */
 
-#if _FFR_REPORTINFO
 /*
 **  DKF_REPORT -- generate a report on failure if possible
 **
@@ -1334,7 +1325,6 @@ dkf_report(msgctx dfc, char *result, char *reason)
 		}
 	}
 }
-#endif /* _FFR_REPORTINFO */
 
 /*
 **  END private section
@@ -2292,8 +2282,7 @@ mlfi_eom(SMFICTX *ctx)
 			return dkf_libstatus(ctx, "dk_eom()", status);
 		}
 
-#ifdef _FFR_QUARANTINE
-# ifdef SMFIF_QUARANTINE
+#ifdef SMFIF_QUARANTINE
 		/* quarantine for "bad" results if requested */
 		if (quarantine &&
 		    (status == DK_STAT_BADSIG ||
@@ -2318,8 +2307,7 @@ mlfi_eom(SMFICTX *ctx)
 				}
 			}
 		}
-# endif /* SMFIF_QUARANTINE */
-#endif /* _FFR_QUARANTINE */
+#endif /* SMFIF_QUARANTINE */
 
 		/* compute and insert the signature, if we're signing */
 		if (dfc->mctx_signing)
@@ -2502,11 +2490,9 @@ mlfi_eom(SMFICTX *ctx)
 			}
 		}
 
-# if _FFR_REPORTINFO
 		if (dfc->mctx_status == DKF_STATUS_BAD &&
 		    dfc->mctx_dk != NULL)
 			dkf_report(dfc, authresult, comment);
-# endif /* _FFR_REPORTINFO */
 	}
 
 	/*
@@ -2519,9 +2505,9 @@ mlfi_eom(SMFICTX *ctx)
 
 		memset(xfhdr, '\0', sizeof xfhdr);
 
-		snprintf(xfhdr, sizeof xfhdr, "%s%s v%s %s %s", DKF_PRODUCT,
+		snprintf(xfhdr, sizeof xfhdr, "%s%s v%s %s %s",
 		         cc->cctx_noleadspc ? " " : "",
-		         DKF_VERSION, hostname,
+		         DKF_PRODUCT, DKF_VERSION, hostname,
 		         dfc->mctx_jobid != NULL ? dfc->mctx_jobid
 		                                : JOBIDUNKNOWN);
 
@@ -2940,9 +2926,7 @@ usage(void)
 #if _FFR_REQUIRED_HEADERS
 	                "-r          \trequire basic RFC2822 header compliance\n"
 #endif /* _FFR_REQUIRED_HEADERS */
-#if _FFR_REPORTINFO
 	                "-R          \tgenerate verification failure reports\n"
-#endif /* _FFR_REPORTINFO */
 	                "-s keyfile  \tlocation of secret key file\n"
 	                "-S selector \tselector to use when signing\n"
 	                "-u userid   \tchange to specified userid\n"
@@ -3012,9 +2996,7 @@ main(int argc, char **argv)
 #if _FFR_FLUSH_HEADERS
 	flushheaders = FALSE;
 #endif /* _FFR_FLUSH_HEADERS */
-#if _FFR_REPORTINFO
 	send_reports = FALSE;
-#endif /* _FFR_REPORTINFO */
 #if _FFR_MULTIPLE_KEYS
 	keyhead = NULL;
 	keytail = NULL;
@@ -3030,9 +3012,7 @@ main(int argc, char **argv)
 	peerlist = NULL;
 	internal = NULL;
 	exignore = NULL;
-#ifdef _FFR_QUARANTINE
 	quarantine = FALSE;
-#endif /* _FFR_QUARANTINE */
 	canon = DK_CANON_SIMPLE;
 	tmo = DEFTIMEOUT;
 	if (DK_DEBUG('t'))
@@ -3165,11 +3145,9 @@ main(int argc, char **argv)
 			pidfile = optarg;
 			break;
 
-#if _FFR_QUARANTINE
 		  case 'q':
 			quarantine = TRUE;
 			break;
-#endif /* _FFR_QUARANTINE */
 
 #if _FFR_REQUIRED_HEADERS
 		  case 'r':
@@ -3177,11 +3155,9 @@ main(int argc, char **argv)
 			break;
 #endif /* _FFR_REQUIRED_HEADERS */
 
-#if _FFR_REPORTINFO
 		  case 'R':
 			send_reports = TRUE;
 			break;
-#endif /* _FFR_REPORTINFO */
 
 		  case 's':
 			if (optarg == NULL || *optarg == '\0')
@@ -4087,9 +4063,7 @@ main(int argc, char **argv)
 		}
 	}
 
-#ifdef _FFR_LOG_SSL_ERRORS
 	ERR_load_crypto_strings();
-#endif /* _FFR_LOG_SSL_ERRORS */
 
 	/* initialize the DomainKeys package */
 	libdk = dk_init(NULL, NULL);
@@ -4111,7 +4085,6 @@ main(int argc, char **argv)
 		(void) dk_options(libdk, DK_OP_SETOPT, &opts);
 	}
 
-#if _FFR_REPORTINFO
 	if (send_reports)
 	{
 		int opts;
@@ -4124,7 +4097,6 @@ main(int argc, char **argv)
 	}
 
 	pthread_mutex_init(&popen_lock, NULL);
-#endif /* _FFR_REPORTINFO */
 
 #ifdef DEBUG
 	return dkf_debug();
