@@ -2,11 +2,11 @@
 **  Copyright (c) 2004-2007 Sendmail, Inc. and its suppliers.
 **    All rights reserved.
 **
-**  $Id: dk.c,v 1.180 2007/04/10 01:42:50 msk Exp $
+**  $Id: dk.c,v 1.181 2007/05/31 02:05:39 msk Exp $
 */
 
 #ifndef lint
-static char dk_c_id[] = "@(#)$Id: dk.c,v 1.180 2007/04/10 01:42:50 msk Exp $";
+static char dk_c_id[] = "@(#)$Id: dk.c,v 1.181 2007/05/31 02:05:39 msk Exp $";
 #endif /* !lint */
 
 /* system includes */
@@ -2531,9 +2531,8 @@ dk_body(DK *dk, u_char *buf, size_t len)
 						for (c = 0;
 						     c < dk->dk_blanks;
 						     c++)
-						{
 							dk_canon(dk, CRLF, 2);
-						}
+
 						dk->dk_blanks = 0;
 
 						dk_canon(dk, wrote, wlen);
@@ -2551,6 +2550,11 @@ dk_body(DK *dk, u_char *buf, size_t len)
 			dk->dk_lastchar = *p;
 			wlen++;
 		}
+
+		/* write what's left */
+		if (wlen > 0)
+			dk_canon(dk, wrote, wlen);
+
 		break;
 
 	  case DK_CANON_NOFWS:
